uniform sampler2D inputTexture;
uniform float time;
uniform vec2 resolution;
uniform float waveFrequency;
uniform float waveAmplitudeFactor;

varying vec2 vUv;
varying vec3 vNormal;  // Receive the normal data

void main() {
    // Distortion intensity varies with the normal's z-component
    float normalInfluence = abs(vNormal.z);  // Use the z-component of the normal for influence

    float waveAmplitude = waveAmplitudeFactor * normalInfluence;  // Modulate amplitude by normal

    // Getting a pseudo-random number
    float rand = fract(sin(dot(vUv.xy ,vec2(12.9898,78.233))) * 43758.5453);
    float smallRandomNumber = rand * 0.1;

    // Modulate time with the random float
    float modulatedTime = time + smallRandomNumber;

    float distortion_y = waveAmplitude * sin(vUv.y * waveFrequency + modulatedTime);
    float distortion_x = waveAmplitude * cos(vUv.x * waveFrequency - modulatedTime);

    // Apply a random effect to distortion
    distortion_x *= fract(sin(dot(vec2(modulatedTime), vec2(12.9898,78.233))) * 43758.5453) * 0.05;
    distortion_y *= fract(sin(dot(vec2(modulatedTime), vec2(12.9898,78.233))) * 43758.5453) * 0.05;

    // Apply the distortion to UV coordinates
    vec2 modUv = vUv;
    modUv.x += distortion_x;  // Distort along the x-axis
    modUv.y += distortion_y;  // Distort along the y-axis

    vec4 textureColor = texture2D(inputTexture, modUv);
    gl_FragColor = vec4(textureColor.rgb, 1.0);
}
